/* opl2java.c */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "opldefs.h"
#include "oplpars.h"

#ifdef __riscos
#include "throwback.h"
#endif

#include "getopt.h"

#ifndef __GNUC__
  char *strdup( const char *s )
  {
    char *t = (char *) malloc( strlen(s)+1 );
    strcpy( t, s );
    return t;
  }
#endif

  char *strndup( const char *s, int n )
  {
    char *t = (char *) malloc( n+1 );
    strncpy( t, s, n );
    t[n] = '\0';
    return t;
  }

#ifndef __GNUC__
  void* __memcpy( void *s1, const void *s2, size_t n )
  {
    return memcpy( s1, s2, n );
  }
#endif

const char* progname = "opl2java";
const char* version = "OPL to Java/C translator  v1.20  " _PLATFORM "  (C) Taurus Software, 1999";

extern int yyparse(void);
extern FILE *yyin;
extern int yydebug;
extern int yy_flex_debug;
int debug_level = 0;
int verbose        = FALSE;
int compile_only   = FALSE;
int translate_only = FALSE;
int generate_debug = FALSE;
int throwback      = FALSE;
char language      = 'j';
int ret;
int errors;

const char* infile  = NULL;
const char* outfile = NULL;

int option;
#ifdef __riscos
extern char* optarg;
#else
char* optarg;
int optind;
#endif

program prog;

void semanticCheck( program * );
void generateCCode( program * );
void generateJavaCode( program *, int );


/* Error throwback support for RISCOS */
#ifdef __riscos

void opl2c_throwback_error( int line, int level, const char* error )
{
	errors++;

	if( throwback )
	{
		if( !errors )
		{
			throwback_start();
			throwback_processing( infile );
		}

		throwback_error( line, level, error );
	}
}

#define THROWBACK_END    if( throwback && errors ) throwback_end();

#else /* __riscos */

void opl2c_throwback_error( int line, int level, const char* error )
{
}

#define THROWBACK_END    ;

#endif /* __riscos */


void usage()
{
	fprintf( stderr, "%s: Usage: oop [-cghtvV] [-d n] [-o file] file\n", progname );
	fprintf( stderr, "      -c      = compile only\n" );
	fprintf( stderr, "      -d n    = debug 1 - parser, 2 - lexer, 3 - both\n" );
	fprintf( stderr, "      -g      = generate debugging info\n" );
	fprintf( stderr, "      -Mj     = generate java application code\n" );
	fprintf( stderr, "      -MJ     = generate java applet code\n" );
	fprintf( stderr, "      -Mc     = generate RISCOS C code\n" );
	fprintf( stderr, "      -o file = output file\n" );
#ifdef __riscos
	fprintf( stderr, "      -t      = throwback on\n" );
#endif
	fprintf( stderr, "      -v      = verbose\n" );
	fprintf( stderr, "      -x      = translate only\n" );
}


int main( int argc, char *argv[] )
{
    fprintf( stderr, "%s\n", version );

	yyin    = NULL;
	yydebug = FALSE;
	yy_flex_debug = FALSE;
	errors = 0;

	if( argc == 1 )
	{
		usage();
		return 1;
	}

#ifdef WIN32
	optind = 1;

	while( (optind < argc) && (argv[optind][0] == '-') )
	{
		option = argv[optind][1];

		if( option != 0 )
			optarg = &argv[optind][2];
#else
	while( (option = getopt( argc, argv, "cd:ghM:o:tvVx" )) != EOF )
	{
#endif

		switch( option )
		{
		case '\0':
			infile = strdup( optarg );
			break;
		case 'V':
			return 0;
		case 'c':
			compile_only = TRUE;
			break;
		case 'g':
			generate_debug = TRUE;
			break;
		case 'd':
			debug_level = atoi( optarg );
			yydebug       = ( debug_level & 1 ? TRUE : FALSE );
			yy_flex_debug = ( debug_level & 2 ? TRUE : FALSE );
			break;
		case 'M':
			switch( optarg[0] )
			{
			case 'j': /* Java Application */
			case 'J': /* Java Applet */
			case 'c': /* RISCOS C */
				language = optarg[0];
				break;
			default:
				fprintf( stderr, "%s: ERROR: Language '%s' not recognised\n", progname, optarg );
				break;
			}
			break;
		case 'o':
			outfile = (const char*) strdup( optarg );
			break;
		case 't':
#ifdef __riscos
			throwback = TRUE;
#else
			fprintf( stderr, "%s: WARNING: Throwback only supported under RISCOS\n", progname );
#endif
			break;
		case 'v':
			/*{
				int n;
				for( n = 0; n < argc; n++ ) 
					fprintf( stderr, "%s ", argv[n] );
				fprintf( stderr, "\n" );
			}*/
			verbose = TRUE;
			break;
		case 'x':
			translate_only = TRUE;
			break;
		case 'h':
		default:
			usage();
			return 1;
		}

#ifdef WIN32
		optind++;
#endif
	}

	if( optind < argc )
		infile = argv[optind];

	if( !infile )
	{
		fprintf( stderr, "%s: No input file specified\n", progname );
		return 1;
	}

	yyin = fopen( infile, "r" );
  
	if( !yyin )
	{
		fprintf( stderr, "%s: unable to open input file, \"%s\"\n", progname, infile );
		return 1;
	}

	prog.pProcs = NULL;
	prog.outputfile = outfile;

	/* shorten sourcefile to just leafname */
	{
		int start, end;

		for( end = strlen( infile ); (end > 0) && (infile[end] != '.'); end-- );
		end--; /* before '.' */
		for( start = end-1; (start > 0) && (infile[start] != '.') && (infile[start] != '/') && (infile[start] != '\\'); start-- );
		start++; /* after sep */

		prog.sourcefile = strdup( &infile[start] );
	}

	ret = yyparse();

	THROWBACK_END;

	fclose( yyin );

	if( ret )
		return 2;

	if( verbose)
		fprintf( stderr, "%s: \"%s\" syntax OK\n", progname, infile );

	switch( language )
	{
	case 'j': /* Java application */
		if( verbose)
			fprintf( stderr, "%s: Generating Java application code...\n", progname );
		generateJavaCode( &prog, 0 );
		break;
	case 'J': /* Java applet */
		if( verbose)
			fprintf( stderr, "%s: Generating Java applet code...\n", progname );
		generateJavaCode( &prog, 1 );
		break;
	case 'c': /* RISCOS C */
		if( verbose)
			fprintf( stderr, "%s: Generating RISCOS C code...\n", progname );
		/* semanticCheck( &prog ); */
		generateCCode( &prog );
		break;
	default:
	    fprintf( stderr, "%s: ERROR: Language not yet implemented\n", progname );
		break;
	}

	return 0;
}
