/* oopdefs.h */
       
#ifndef __oopdefs_h
#define __oopdefs_h



void yyerror( char* s );



#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif



/*****************************************************************************
 *****************************************************************************
 **                  Structures which form parse tree                       **
 *****************************************************************************
 *****************************************************************************/


/*****************************************************************************
 * Identifer
 *****************************************************************************/

typedef enum { untyped_id=0, integer_id, floating_point_id, string_id, long_integer_id } identType;

typedef struct
{
  char                  *name;
  identType              type;
  struct expressionList *pArrays;
} ident;

ident* new_ident( char* n, identType t, struct expressionList *pa );

typedef struct identList {
  ident            *head;
  struct identList *tail;
} identList;

identList *new_identList( ident *el, identList *l );


/*****************************************************************************
 * Procedure or Function
 *****************************************************************************/

typedef struct
{
  ident                *name; 
  struct identList     *pArgs;
  struct statementList *pStmts;
} procedure;

procedure* new_procedure( ident* n, identList* pa, struct statementList *ps );

typedef struct procedureList {
  procedure            *head;
  struct procedureList *tail;
} procedureList;

procedureList *new_procedureList( procedure *el, procedureList *l );


/*****************************************************************************
 * Program
 *****************************************************************************/

typedef struct
{
	struct procedureList *pProcs;
	const char *sourcefile;
	const char *outputfile; 
} program;

program* new_program( procedureList *pp );


/******************************************************************************
 * Various forms of Statement
 *****************************************************************************/

typedef enum
{
  if_stmt,
  while_stmt,
  do_stmt,
  assignment_stmt,
  call_stmt,
  global_stmt,
  local_stmt,
  return_stmt
} stmtType;

typedef struct
{
  stmtType kind;
  union
  {
    struct
    {
      struct expression      *condition;
      struct statementList   *pThen;
      struct statementList   *pElse;
    } if_part;      
    struct
    {
      struct expression      *condition;
      struct statementList   *pBody;
    } while_and_do_part;      
    struct
    {
      ident                  *name;
      struct expression      *value;
    } assignment_part;      
    struct
    {
      identList             *pVars;
    } global_and_local_part;      
    struct
    {
      ident                 *name;
      struct expressionList *pArgs;
      int                    opl_call;
    } call_part;
    struct
    {
      struct expression     *expr;
    } return_part;
  } data;
} statement;

statement* new_if_statement( struct expression *c,
  struct statementList *pt, struct statementList *pe );

statement* new_while_statement( struct expression *c,
  struct statementList *pb );

statement* new_do_statement( struct expression *c, struct statementList *pb );

statement* new_assignment_statement( ident *n, struct expression *v );

statement* new_global_statement( identList *pi );

statement* new_local_statement( identList *pi );

statement* new_call_statement( ident *n, struct expressionList *pa, int opl );

statement* new_return_statement( struct expression *e );

typedef struct statementList {
  statement            *head;
  struct statementList *tail;
} statementList;

statementList *new_statementList( statement *el, statementList *l );


/*****************************************************************************
 * Various forms of Expression
 *****************************************************************************/

typedef enum
{
  variable_expr, integer_expr, hexint_expr, character_expr,
  floating_point_expr, long_integer_expr, string_expr,
  addition_expr, subtraction_expr, multiplication_expr,
  division_expr, and_expr, or_expr, equality_expr,
  inequality_expr, lessthan_expr, greaterthan_expr,
  lessequal_expr, greaterequal_expr, uminus_expr,
  parenthesis_expr, call_expr, power_expr
} exprKind;

typedef enum
{
	untyped_type=untyped_id, integer_type=integer_id,
    float_type=floating_point_id, string_type=string_id,
    long_type=long_integer_id, boolean_type
} exprType;

typedef struct expression
{
  exprKind    kind;
  exprType    type;
  union
  {
    ident             *variable_part;
    int                integer_part;
    double             floating_point_part;
    long               long_integer_part;
    const char        *string_part;
    struct
    {
      struct expression      *left;
      struct expression      *right;
    } binary_op_part;
    struct
    {
      struct expression      *expr;
    } unary_op_part;
    struct
    {
      ident                 *name;
      struct expressionList *pArgs;
      int                    opl_call;
    } call_part;
  } data;
} expression;

expression* new_variable_expr( ident *v );
expression* new_integer_expr( int k, int n );
/*expression* new_floating_point_expr(); - not needed? */
/*expression* new_long_integer_expr(); - not needed? */
expression* new_string_expr( const char *s );

expression* new_binary_op_expression( exprKind k, expression* l, expression* r );
expression* new_unary_op_expression( exprKind k, expression* e );
expression* new_call_expression( ident* n, struct expressionList* pa, int opl );

typedef struct expressionList {
  expression            *head;
  struct expressionList *tail;
} expressionList;

expressionList *new_expressionList( expression *el, expressionList *l );

const char* typeToString( exprType t );

/*****************************************************************************
 *****************************************************************************/

extern program prog;



#endif /* __oopdefs_h */
